// license:BSD-3-Clause
// copyright-holders:Nicola Salmoria
/*************************************************************************

    Zero Hour / Red Clash

*************************************************************************/
#ifndef MAME_INCLUDES_REDCLASH_H
#define MAME_INCLUDES_REDCLASH_H

#pragma once

#include "video/ladybug.h"


// redclash/zerohour
class redclash_state : public driver_device
{
public:
	redclash_state(const machine_config &mconfig, device_type type, const char *tag)
		: driver_device(mconfig, type, tag)
		, m_videoram(*this, "videoram")
		, m_spriteram(*this, "spriteram")
		, m_maincpu(*this, "maincpu")
		, m_palette(*this, "palette")
		, m_gfxdecode(*this, "gfxdecode")
		, m_stars(*this, "stars")
	{ }

	DECLARE_INPUT_CHANGED_MEMBER(left_coin_inserted);
	DECLARE_INPUT_CHANGED_MEMBER(right_coin_inserted);
	DECLARE_DRIVER_INIT(redclash);
	void redclash(machine_config &config);
	void zerohour(machine_config &config);

protected:
	DECLARE_WRITE_LINE_MEMBER(screen_vblank_redclash);
	DECLARE_WRITE8_MEMBER(redclash_videoram_w);
	DECLARE_WRITE8_MEMBER(redclash_gfxbank_w);
	DECLARE_WRITE8_MEMBER(redclash_flipscreen_w);
	DECLARE_WRITE8_MEMBER(irqack_w);
	DECLARE_WRITE8_MEMBER(redclash_star_reset_w);
	template <unsigned B> DECLARE_WRITE8_MEMBER(redclash_star_w);
	DECLARE_PALETTE_INIT(redclash);
	TILE_GET_INFO_MEMBER(get_fg_tile_info);

	virtual void machine_start() override;
	virtual void machine_reset() override;
	virtual void video_start() override;
	uint32_t screen_update_redclash(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
	void redclash_draw_sprites(bitmap_ind16 &bitmap, const rectangle &cliprect);
	void redclash_draw_bullets(bitmap_ind16 &bitmap, const rectangle &cliprect);

	void redclash_map(address_map &map);
	void zerohour_map(address_map &map);

private:
	required_shared_ptr<uint8_t> m_videoram;
	required_shared_ptr<uint8_t> m_spriteram;
	required_device<cpu_device> m_maincpu;
	required_device<palette_device> m_palette;
	required_device<gfxdecode_device> m_gfxdecode;
	required_device<zerohour_stars_device> m_stars;

	tilemap_t   *m_fg_tilemap;
	int         m_gfxbank;   // redclash only
};

#endif // MAME_INCLUDES_REDCLASH_H
